# Reparatur unzulaessiger Loesungen

# auf den Rand setzen
f_margin = function(x, a, b) {
  ##### CODE #####
}

# Modulo
f_modulo = function(x, a, b) {
  ##### CODE #####
}

# Achsenspiegelung
f_axis = function(x, a, b) {
  ##### CODE #####
}

# input:
# f: function to minimize
# lambda: number of children
# parents: initial parent
# evals: budget of function evaluations
# lower: lower bound (same for all dimensions)
# upper: upper bound (same for all dimensions)
# sigma0: initial step size
# tau: standard deviation of normal distribution (Schwefel)
# correct: choice of repair mechanism (options: "margin", "modulo", "axis")
one_lambda_ea_r = function(f, lambda, parent, evals, lower, upper, sigma0, tau, correct) {
  # check input:
  assert_function(f, arg = "x")
  assert_count(lambda, positive = TRUE)
  assert_numeric(parent, lower = lower, upper = upper, min.len = 1, finite = TRUE)
  assert_count(evals, positive = TRUE)
  assert_number(lower, finite = TRUE)
  assert_number(upper, lower = lower, finite = TRUE)
  assert_number(sigma0, finite = TRUE)
  assert_number(tau, lower = 0, finite = TRUE)
  assert_choice(correct, c("margin", "modulo", "axis"))

  # initialize parent:
  n = length(parent)
  sigma = sigma0
  fitness_p = f(parent)
  evals_left = evals - 1

  while(evals_left > 0) {
    # schwefel method:
    sigma = sigma * exp(rnorm(n = 1, mean = 0, sd = tau))
    lambda = min(lambda, evals_left)
    children = parent + sigma * matrix(rnorm(lambda * n), nrow = n)

    # check if all children are valid and correct values if necessary:
    f_correct = switch(correct,
      margin = f_margin,
      modulo = f_modulo,
      axis = f_axis)
    for(i in 1:nrow(children)) {
      children[i, ] = sapply(children[i, ], function(x) {
        corrected = f_correct(x, lower, upper)
        # repair until feasible
        while (corrected < lower || corrected > upper) {
          corrected = f_correct(corrected, lower, upper)
        }
        return(corrected)
      })
    }

    # evaluate children:
    fitness_c = apply(children, 2, f)
    evals_left = evals_left - lambda

    # selection for next generation:
    min_ind = which.min(c(fitness_p, fitness_c))
    parent = rbind(parent, t(children))[min_ind, ]
    fitness_p = c(fitness_p, fitness_c)[min_ind]
  }
  return(list(x = parent, fx = fitness_p))
}
